@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 """
 General Utilities
 (part of web.py)
@@ -8,39 +7,26 @@ __all__ = [
   "Storage", "storage", "storify", 
   "iters", 
   "rstrips", "lstrips", "strips", 
-  "safeunicode", "safestr", "utf8",
   "TimeoutError", "timelimit",
   "Memoize", "memoize",
   "re_compile", "re_subm",
-  "group", "uniq", "iterview",
+  "group",
   "IterBetter", "iterbetter",
   "dictreverse", "dictfind", "dictfindall", "dictincr", "dictadd",
   "listget", "intget", "datestr",
-  "numify", "denumify", "commify", "dateify",
-  "nthstr",
+  "numify", "denumify", "dateify",
   "CaptureStdout", "capturestdout", "Profile", "profile",
   "tryall",
-  "ThreadedDict", "threadeddict",
+  "ThreadedDict",
   "autoassign",
   "to36",
-  "safemarkdown",
-  "sendmail"
+  "safemarkdown"
 ]
 
-import re, sys, time, threading, itertools
-
-try:
-    import subprocess
-except ImportError: 
-    subprocess = None
-
+import re, sys, time, threading
 try: import datetime
 except ImportError: pass
 
-try: set
-except NameError:
-    from sets import Set as set
-
 class Storage(dict):
     """
     A Storage object is like a dictionary except `obj.foo` can be used
@@ -66,16 +52,16 @@ class Storage(dict):
             return self[key]
         except KeyError, k:
             raise AttributeError, k
-    
+
     def __setattr__(self, key, value): 
         self[key] = value
-    
+
     def __delattr__(self, key):
         try:
             del self[key]
         except KeyError, k:
             raise AttributeError, k
-    
+
     def __repr__(self):     
         return '<Storage ' + dict.__repr__(self) + '>'
 
@@ -112,28 +98,13 @@ def storify(mapping, *requireds, **defaults):
         <Storage {'value': 1}>
         >>> storify({}, a={}).a
         {}
-        
-    Optionally, keyword parameter `_unicode` can be passed to convert all values to unicode.
     
-        >>> storify({'x': 'a'}, _unicode=True)
-        <Storage {'x': u'a'}>
-        >>> storify({'x': storage(value='a')}, x={}, _unicode=True)
-        <Storage {'x': <Storage {'value': 'a'}>}>
-        >>> storify({'x': storage(value='a')}, _unicode=True)
-        <Storage {'x': u'a'}>
     """
-    _unicode = defaults.pop('_unicode', False)
-    def unicodify(s):
-        if _unicode and isinstance(s, str): return safeunicode(s)
-        else: return s
-        
     def getvalue(x):
-        if hasattr(x, 'file') and hasattr(x, 'value'):
+        if hasattr(x, 'value'):
             return x.value
-        elif hasattr(x, 'value'):
-            return unicodify(x.value)
         else:
-            return unicodify(x)
+            return x
     
     stor = Storage()
     for key in requireds + tuple(mapping.keys()):
@@ -163,15 +134,12 @@ iters = [list, tuple]
 import __builtin__
 if hasattr(__builtin__, 'set'):
     iters.append(set)
-if hasattr(__builtin__, 'frozenset'):
-    iters.append(set)
-if sys.version_info < (2,6): # sets module deprecated in 2.6
-    try:
-        from sets import Set
-        iters.append(Set)
-    except ImportError: 
-        pass
-    
+try: 
+    from sets import Set
+    iters.append(Set)
+except ImportError: 
+    pass
+
 class _hack(tuple): pass
 iters = _hack(iters)
 iters.__doc__ = """
@@ -211,8 +179,7 @@ def lstrips(text, remove):
     return _strips('l', text, remove)
 
 def strips(text, remove):
-    """
-    removes the string `remove` from the both sides of `text`
+    """removes the string `remove` from the both sides of `text`
 
         >>> strips("foobarfoo", "foo")
         'bar'
@@ -220,50 +187,6 @@ def strips(text, remove):
     """
     return rstrips(lstrips(text, remove), remove)
 
-def safeunicode(obj, encoding='utf-8'):
-    r"""
-    Converts any given object to unicode string.
-    
-        >>> safeunicode('hello')
-        u'hello'
-        >>> safeunicode(2)
-        u'2'
-        >>> safeunicode('\xe1\x88\xb4')
-        u'\u1234'
-    """
-    if isinstance(obj, unicode):
-        return obj
-    elif isinstance(obj, str):
-        return obj.decode(encoding)
-    else:
-        if hasattr(obj, '__unicode__'):
-            return unicode(obj)
-        else:
-            return str(obj).decode(encoding)
-    
-def safestr(obj, encoding='utf-8'):
-    r"""
-    Converts any given object to utf-8 encoded string. 
-    
-        >>> safestr('hello')
-        'hello'
-        >>> safestr(u'\u1234')
-        '\xe1\x88\xb4'
-        >>> safestr(2)
-        '2'
-    """
-    if isinstance(obj, unicode):
-        return obj.encode('utf-8')
-    elif isinstance(obj, str):
-        return obj
-    elif hasattr(obj, 'next') and hasattr(obj, '__iter__'): # iterator
-        return itertools.imap(safestr, obj)
-    else:
-        return str(obj)
-
-# for backward-compatibility
-utf8 = safestr
-    
 class TimeoutError(Exception): pass
 def timelimit(timeout):
     """
@@ -317,80 +240,32 @@ def timelimit(timeout):
 class Memoize:
     """
     'Memoizes' a function, caching its return values for each input.
-    If `expires` is specified, values are recalculated after `expires` seconds.
-    If `background` is specified, values are recalculated in a separate thread.
     
-        >>> calls = 0
-        >>> def howmanytimeshaveibeencalled():
-        ...     global calls
-        ...     calls += 1
-        ...     return calls
-        >>> fastcalls = memoize(howmanytimeshaveibeencalled)
-        >>> howmanytimeshaveibeencalled()
-        1
-        >>> howmanytimeshaveibeencalled()
-        2
-        >>> fastcalls()
-        3
-        >>> fastcalls()
-        3
         >>> import time
-        >>> fastcalls = memoize(howmanytimeshaveibeencalled, .1, background=False)
-        >>> fastcalls()
-        4
-        >>> fastcalls()
-        4
-        >>> time.sleep(.2)
-        >>> fastcalls()
-        5
-        >>> def slowfunc():
-        ...     time.sleep(.1)
-        ...     return howmanytimeshaveibeencalled()
-        >>> fastcalls = memoize(slowfunc, .2, background=True)
-        >>> fastcalls()
-        6
-        >>> timelimit(.05)(fastcalls)()
-        6
-        >>> time.sleep(.2)
-        >>> timelimit(.05)(fastcalls)()
-        6
-        >>> timelimit(.05)(fastcalls)()
-        6
-        >>> time.sleep(.2)
-        >>> timelimit(.05)(fastcalls)()
-        7
-        >>> fastcalls = memoize(slowfunc, None, background=True)
-        >>> threading.Thread(target=fastcalls).start()
-        >>> time.sleep(.01)
-        >>> fastcalls()
-        9
+        >>> def meaningoflife():
+        ...     time.sleep(.2)
+        ...     return 42
+        >>> fastlife = memoize(meaningoflife)
+        >>> meaningoflife()
+        42
+        >>> timelimit(.1)(meaningoflife)()
+        Traceback (most recent call last):
+            ...
+        TimeoutError: took too long
+        >>> fastlife()
+        42
+        >>> timelimit(.1)(fastlife)()
+        42
+    
     """
-    def __init__(self, func, expires=None, background=True): 
+    def __init__(self, func): 
         self.func = func
         self.cache = {}
-        self.expires = expires
-        self.background = background
-        self.running = {}
-    
     def __call__(self, *args, **keywords):
         key = (args, tuple(keywords.items()))
-        if not self.running.get(key):
-            self.running[key] = threading.Lock()
-        def update(block=False):
-            if self.running[key].acquire(block):
-                try:
-                    self.cache[key] = (self.func(*args, **keywords), time.time())
-                finally:
-                    self.running[key].release()
-        
         if key not in self.cache: 
-            update(block=True)
-        elif self.expires and (time.time() - self.cache[key][1]) > self.expires:
-            if self.background:
-                threading.Thread(target=update).start()
-            else:
-                update()
-        return self.cache[key][0]
+            self.cache[key] = self.func(*args, **keywords)
+        return self.cache[key]
 
 memoize = Memoize
 
@@ -427,84 +302,11 @@ def group(seq, size):
 
         >>> list(group([1,2,3,4], 2))
         [[1, 2], [3, 4]]
-        >>> list(group([1,2,3,4,5], 2))
-        [[1, 2], [3, 4], [5]]
     """
-    def take(seq, n):
-        for i in xrange(n):
-            yield seq.next()
-
     if not hasattr(seq, 'next'):  
         seq = iter(seq)
     while True: 
-        x = list(take(seq, size))
-        if x:
-            yield x
-        else:
-            break
-
-def uniq(seq):
-   """
-   Removes duplicate elements from a list.
-
-       >>> uniq([1,2,3,1,4,5,6])
-       [1, 2, 3, 4, 5, 6]
-   """
-   seen = set()
-   result = []
-   for item in seq:
-       if item in seen: continue
-       seen.add(item)
-       result.append(item)
-   return result
-
-def iterview(x):
-   """
-   Takes an iterable `x` and returns an iterator over it
-   which prints its progress to stderr as it iterates through.
-   """
-   WIDTH = 70
-
-   def plainformat(n, lenx):
-       return '%5.1f%% (%*d/%d)' % ((float(n)/lenx)*100, len(str(lenx)), n, lenx)
-
-   def bars(size, n, lenx):
-       val = int((float(n)*size)/lenx + 0.5)
-       if size - val:
-           spacing = ">" + (" "*(size-val))[1:]
-       else:
-           spacing = ""
-       return "[%s%s]" % ("="*val, spacing)
-
-   def eta(elapsed, n, lenx):
-       if n == 0:
-           return '--:--:--'
-       if n == lenx:
-           secs = int(elapsed)
-       else:
-           secs = int((elapsed/n) * (lenx-n))
-       mins, secs = divmod(secs, 60)
-       hrs, mins = divmod(mins, 60)
-
-       return '%02d:%02d:%02d' % (hrs, mins, secs)
-
-   def format(starttime, n, lenx):
-       out = plainformat(n, lenx) + ' '
-       if n == lenx:
-           end = '     '
-       else:
-           end = ' ETA '
-       end += eta(time.time() - starttime, n, lenx)
-       out += bars(WIDTH - len(out) - len(end), n, lenx)
-       out += end
-       return out
-
-   starttime = time.time()
-   lenx = len(x)
-   for n, y in enumerate(x):
-       sys.stderr.write('\r' + format(starttime, n, lenx))
-       yield y
-   sys.stderr.write('\r' + format(starttime, n+1, lenx) + '\n')
+        yield [seq.next() for i in xrange(size)]
 
 class IterBetter:
     """
@@ -543,16 +345,10 @@ class IterBetter:
             return self.i.next()
         except StopIteration: 
             raise IndexError, str(i)
-            
-    def __nonzero__(self):
-        return len(self) != 0
-        
 iterbetter = IterBetter
 
 def dictreverse(mapping):
     """
-    Returns a new dictionary with keys and values swapped.
-    
         >>> dictreverse({1: 2, 3: 4})
         {2: 1, 4: 3}
     """
@@ -608,17 +404,17 @@ def dictincr(dictionary, element):
     dictionary[element] += 1
     return dictionary[element]
 
-def dictadd(*dicts):
+def dictadd(dict_a, dict_b):
     """
-    Returns a dictionary consisting of the keys in the argument dictionaries.
-    If they share a key, the value from the last argument is used.
+    Returns a dictionary consisting of the keys in `a` and `b`.
+    If they share a key, the value from b is used.
     
         >>> dictadd({1: 0, 2: 0}, {2: 1, 3: 1})
         {1: 0, 2: 1, 3: 1}
     """
     result = {}
-    for dct in dicts:
-        result.update(dct)
+    result.update(dict_a)
+    result.update(dict_b)
     return result
 
 def listget(lst, ind, default=None):
@@ -676,8 +472,6 @@ def datestr(then, now=None):
         'January  1, 1969'
         >>> datestr(datetime(1970, 6, 1), now=d)
         'June  1, 1970'
-        >>> datestr(None)
-        ''
     """
     def agohence(n, what, divisor=None):
         if divisor: n = n // divisor
@@ -693,15 +487,7 @@ def datestr(then, now=None):
 
     oneday = 24 * 60 * 60
 
-    if not then: return ""
     if not now: now = datetime.datetime.utcnow()
-    if type(now).__name__ == "DateTime":
-        now = datetime.datetime.fromtimestamp(now)
-    if type(then).__name__ == "DateTime":
-        then = datetime.datetime.fromtimestamp(then)
-    elif type(then).__name__ == "date":
-        then = datetime.datetime(then.year, then.month, then.day)
-
     delta = now - then
     deltaseconds = int(delta.days * oneday + delta.seconds + delta.microseconds * 1e-06)
     deltadays = abs(deltaseconds) // oneday
@@ -737,11 +523,9 @@ def numify(string):
     
         >>> numify('800-555-1212')
         '8005551212'
-        >>> numify('800.555.1212')
-        '8005551212'
     
     """
-    return ''.join([c for c in str(string) if c.isdigit()])
+    return ''.join(c for c in str(string).split('.')[0] if c.isdigit())
 
 def denumify(string, pattern):
     """
@@ -761,91 +545,12 @@ def denumify(string, pattern):
             out.append(c)
     return ''.join(out)
 
-def commify(n):
-    """
-    Add commas to an integer `n`.
-
-        >>> commify(1)
-        '1'
-        >>> commify(123)
-        '123'
-        >>> commify(1234)
-        '1,234'
-        >>> commify(1234567890)
-        '1,234,567,890'
-        >>> commify(123.0)
-        '123.0'
-        >>> commify(1234.5)
-        '1,234.5'
-        >>> commify(1234.56789)
-        '1,234.56789'
-        >>> commify('%.2f' % 1234.5)
-        '1,234.50'
-        >>> commify(None)
-        >>>
-
-    """
-    if n is None: return None
-    n = str(n)
-    if '.' in n:
-        dollars, cents = n.split('.')
-    else:
-        dollars, cents = n, None
-
-    r = []
-    for i, c in enumerate(str(dollars)[::-1]):
-        if i and (not (i % 3)):
-            r.insert(0, ',')
-        r.insert(0, c)
-    out = ''.join(r)
-    if cents:
-        out += '.' + cents
-    return out
-
 def dateify(datestring):
     """
     Formats a numified `datestring` properly.
     """
     return denumify(datestring, "XXXX-XX-XX XX:XX:XX")
 
-
-def nthstr(n):
-    """
-    Formats an ordinal.
-    Doesn't handle negative numbers.
-
-        >>> nthstr(1)
-        '1st'
-        >>> nthstr(0)
-        '0th'
-        >>> [nthstr(x) for x in [2, 3, 4, 5, 10, 11, 12, 13, 14, 15]]
-        ['2nd', '3rd', '4th', '5th', '10th', '11th', '12th', '13th', '14th', '15th']
-        >>> [nthstr(x) for x in [91, 92, 93, 94, 99, 100, 101, 102]]
-        ['91st', '92nd', '93rd', '94th', '99th', '100th', '101st', '102nd']
-        >>> [nthstr(x) for x in [111, 112, 113, 114, 115]]
-        ['111th', '112th', '113th', '114th', '115th']
-
-    """
-    
-    assert n >= 0
-    if n % 100 in [11, 12, 13]: return '%sth' % n
-    return {1: '%sst', 2: '%snd', 3: '%srd'}.get(n % 10, '%sth') % n
-
-def cond(predicate, consequence, alternative=None):
-    """
-    Function replacement for if-else to use in expressions.
-        
-        >>> x = 2
-        >>> cond(x % 2 == 0, "even", "odd")
-        'even'
-        >>> cond(x % 2 == 0, "even", "odd") + '_row'
-        'even_row'
-    """
-    if predicate:
-        return consequence
-    else:
-        return alternative
-
 class CaptureStdout:
     """
     Captures everything `func` prints to stdout and returns it instead.
@@ -887,35 +592,21 @@ class Profile:
     def __init__(self, func): 
         self.func = func
     def __call__(self, *args): ##, **kw):   kw unused
-        import hotshot, hotshot.stats, os, tempfile ##, time already imported
-        f, filename = tempfile.mkstemp()
-        os.close(f)
-        
-        prof = hotshot.Profile(filename)
+        import hotshot, hotshot.stats, tempfile ##, time already imported
+        temp = tempfile.NamedTemporaryFile()
+        prof = hotshot.Profile(temp.name)
 
         stime = time.time()
         result = prof.runcall(self.func, *args)
         stime = time.time() - stime
-        prof.close()
 
-        import cStringIO
-        out = cStringIO.StringIO()
-        stats = hotshot.stats.load(filename)
-        stats.stream = out
+        prof.close()
+        stats = hotshot.stats.load(temp.name)
         stats.strip_dirs()
         stats.sort_stats('time', 'calls')
-        stats.print_stats(40)
-        stats.print_callers()
-
         x =  '\n\ntook '+ str(stime) + ' seconds\n'
-        x += out.getvalue()
-
-        # remove the tempfile
-        try:
-            os.remove(filename)
-        except IOError:
-            pass
-            
+        x += capturestdout(stats.print_stats)(40)
+        x += capturestdout(stats.print_callers)()
         return result, x
 
 profile = Profile
@@ -973,47 +664,34 @@ def tryall(context, prefix=None):
     print 'results:'
     for (key, value) in results.iteritems():
         print ' '*2, str(key)+':', value
-        
+
 class ThreadedDict:
     """
-    Thread local storage.
-    
-        >>> d = ThreadedDict()
-        >>> d.x = 1
-        >>> d.x
-        1
-        >>> import threading
-        >>> def f(): d.x = 2
-        ...
-        >>> t = threading.Thread(target=f)
-        >>> t.start()
-        >>> t.join()
-        >>> d.x
-        1
+    Takes a dictionary that maps threads to objects. 
+    When a thread tries to get or set an attribute or item 
+    of the threadeddict, it passes it on to the object 
+    for that thread in dictionary.
     """
-    def __getattr__(self, key):
-        return getattr(self._getd(), key)
+    def __init__(self, dictionary): 
+        self.__dict__['_ThreadedDict__d'] = dictionary
 
-    def __setattr__(self, key, value):
-        return setattr(self._getd(), key, value)
+    def __getattr__(self, attr): 
+        return getattr(self.__d[threading.currentThread()], attr)
 
-    def __delattr__(self, key):
-        return delattr(self._getd(), key)
+    def __getitem__(self, item): 
+        return self.__d[threading.currentThread()][item]
+
+    def __setattr__(self, attr, value):
+        if attr == '__doc__':
+            self.__dict__[attr] = value
+        else:
+            return setattr(self.__d[threading.currentThread()], attr, value)
+
+    def __setitem__(self, item, value): 
+        self.__d[threading.currentThread()][item] = value
 
     def __hash__(self): 
-        return id(self)
-
-    def _getd(self):
-        t = threading.currentThread()
-        if not hasattr(t, '_d'):
-            # using __dict__ of thread as thread local storage
-            t._d = {}
-
-        # there could be multiple instances of ThreadedDict.
-        # use self as key
-        if self not in t._d:
-            t._d[self] = storage()
-        return t._d[self]
+        return hash(self.__d[threading.currentThread()])
 
 threadeddict = ThreadedDict
 
@@ -1079,105 +757,6 @@ def safemarkdown(text):
         text = markdown(text)
         return text
 
-def sendmail(from_address, to_address, subject, message, headers=None, **kw):
-    """
-    Sends the email message `message` with mail and envelope headers
-    for from `from_address_` to `to_address` with `subject`. 
-    Additional email headers can be specified with the dictionary 
-    `headers.
-
-    If `web.config.smtp_server` is set, it will send the message
-    to that SMTP server. Otherwise it will look for 
-    `/usr/sbin/sendmail`, the typical location for the sendmail-style
-    binary. To use sendmail from a different path, set `web.config.sendmail_path`.
-    """
-    try:
-        import webapi
-    except ImportError:
-        webapi = Storage(config=Storage())
-    
-    if headers is None: headers = {}
-    
-    cc = kw.get('cc', [])
-    bcc = kw.get('bcc', [])
-    
-    def listify(x):
-        if not isinstance(x, list):
-            return [safestr(x)]
-        else:
-            return [safestr(a) for a in x]
-
-    from_address = safestr(from_address)
-
-    to_address = listify(to_address)
-    cc = listify(cc)
-    bcc = listify(bcc)
-
-    recipients = to_address + cc + bcc
-    
-    headers = dictadd({
-      'MIME-Version': '1.0',
-      'Content-Type': 'text/plain; charset=UTF-8',
-      'Content-Disposition': 'inline',
-      'From': from_address,
-      'To': ", ".join(to_address),
-      'Subject': subject
-    }, headers)
-
-    if cc:
-        headers['Cc'] = ", ".join(cc)
-    
-    import email.Utils
-    from_address = email.Utils.parseaddr(from_address)[1]
-    recipients = [email.Utils.parseaddr(r)[1] for r in recipients]
-    message = ('\n'.join([safestr('%s: %s' % x) for x in headers.iteritems()])
-      + "\n\n" +  safestr(message))
-
-    if webapi.config.get('smtp_server'):
-        server = webapi.config.get('smtp_server')
-        port = webapi.config.get('smtp_port', 0)
-        username = webapi.config.get('smtp_username') 
-        password = webapi.config.get('smtp_password')
-        debug_level = webapi.config.get('smtp_debuglevel', None)
-        starttls = webapi.config.get('smtp_starttls', False)
-
-        import smtplib
-        smtpserver = smtplib.SMTP(server, port)
-
-        if debug_level:
-            smtpserver.set_debuglevel(debug_level)
-
-        if starttls:
-            smtpserver.ehlo()
-            smtpserver.starttls()
-            smtpserver.ehlo()
-
-        if username and password:
-            smtpserver.login(username, password)
-
-        smtpserver.sendmail(from_address, recipients, message)
-        smtpserver.quit()
-    else:
-        sendmail = webapi.config.get('sendmail_path', '/usr/sbin/sendmail')
-        
-        assert not from_address.startswith('-'), 'security'
-        for r in recipients:
-            assert not r.startswith('-'), 'security'
-                
-        cmd = [sendmail, '-f', from_address] + recipients
-
-        if subprocess:
-            p = subprocess.Popen(cmd, stdin=subprocess.PIPE)
-            p.stdin.write(message)
-            p.stdin.close()
-            p.wait()
-        else:
-            import os
-            i, o = os.popen2(cmd)
-            i.write(message)
-            i.close()
-            o.close()
-            del i, o
 
 if __name__ == "__main__":
     import doctest
